<?php

namespace Drupal\vegas;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\path_alias\AliasManagerInterface;
use Drupal\Core\Path\PathMatcherInterface;
use Drupal\Core\Routing\AdminContext;
use Drupal\Core\Url;

/**
 * VegasPathService Class.
 *
 * Class to handle th Vegas configuration Entity path settings.
 */
class VegasPathService {

  /**
   * Provides an interface for URL path matchers.
   *
   * @var \Drupal\Core\Path\PathMatcherInterface
   */
  protected $pathMatcher;

  /**
   * Find an alias for a path and vice versa.
   *
   * @var \Drupal\path_alias\AliasManagerInterface
   */
  protected $aliasManager;

  /**
   * Defines the immutable configuration object.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */

  protected $config;

  /**
   * Provides a helper class to determine whether the route is an admin one.
   *
   * @var \Drupal\Core\Routing\AdminContext
   */
  protected $adminContext;

  /**
   * An array of paths that were already checked and their match status.
   *
   * @var array
   */
  protected $matches = [];

  /**
   * An array of vegas configuration entities.
   *
   * @var array
   */
  protected $vegasentity = [];

  /**
   * Constructs a new vegas service.
   *
   * @param \Drupal\Core\Path\PathMatcherInterface $path_matcher
   *   Patch matcher services for comparing the halloween patterns.
   * @param \Drupal\path_alias\AliasManagerInterface $alias_manager
   *   Alias manager so we can also test path aliases.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory so we can get the halloween settings.
   * @param \Drupal\Core\Routing\AdminContext $admin_context
   *   Determines whether the active route is an admin one.
   */
  public function __construct(PathMatcherInterface $path_matcher, AliasManagerInterface $alias_manager, ConfigFactoryInterface $config_factory, AdminContext $admin_context) {
    $this->pathMatcher = $path_matcher;
    $this->aliasManager = $alias_manager;
    $this->vegasentity = $this->getMultipleVegasConfig();
    $this->adminContext = $admin_context;
  }

  /**
   * Load multiple vegas configuration entities.
   */
  public function getMultipleVegasConfig() {
    $vegasentity = \Drupal::entityTypeManager()->getStorage('vegas_config')->loadMultiple();
    return $vegasentity;
  }

  /**
   * This function checks both internal paths and aliased paths.
   *
   * @param \Drupal\Core\Url $url
   *   The Url object.
   *
   * @return bool
   *   TRUE if it matches the given rules.
   */
  public function isVegasPath(Url $url) {

    if ($url->isExternal()) {
      return FALSE;
    }

    if ($this->adminContext->isAdminRoute()) {
      return FALSE;
    }

    // Like core is trying to force us into.
    $path = strtolower($url->toString());

    // We filter out empty paths because some modules (such as Media) use
    // theme_link() to generate links with empty paths and we filter out paths
    // that do not start with a /, such as #hash-only URLs.
    if (empty($path) || $path[0] !== '/') {
      return FALSE;
    }

    // Remove the base path.
    if ($base_path = \Drupal::request()->getBasePath()) {
      $path = substr($path, strlen($base_path));
    }

    // Check the cache, see if we've handled this before.
    if (isset($this->matches[$path])) {
      return $this->matches[$path];
    }

    // Normalise the patterns as well so they match the normalised paths.
    // $patterns = strtolower($this->config->get('tb_halloween_patterns'));.
    foreach ($this->vegasentity as $entity) {
      $patterns = $entity->get('config_vegas_path');

      // Check for internal paths first which is much quicker than alias lookup.
      if ($this->pathMatcher->matchPath($path, $patterns)) {
        $this->matches[$path] = TRUE;
        return $entity;
      }
      else {
        // Now check for aliases paths.
        $aliased_path = strtolower($this->aliasManager->getAliasByPath($path));
        if ($path != $aliased_path && $this->pathMatcher->matchPath($aliased_path, $patterns)) {
          $this->matches[$path] = TRUE;
          return $entity;
        }
      }
    }
  }

}
